local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/woodenFloors/baseOverHanging.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register materials
StoneGate:registerAssetId("models/woodenFloors/baseOverHanging.fbx/Materials/murChateau", StoneGateSetPrefix .. "_OVER_HANGING_STONE_MANOR_WALL_MATERIAL")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
                IsVisibleWhenBuilt = false,  -- to mask the green diamond
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

function registerRootPart(_nameFbxFile, _nameModel, _nameFbx, _nameAsset, _stoneCost)

    -- Register Keep prefab nodes
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_0_".. _nameFbx .. "_RootPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOT_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_1_".. _nameFbx .. "_Tiling1PPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1P_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_2_".. _nameFbx .. "_Tiling2PPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2P_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _nameFbx .. "_TopPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TOP_PREFAB")
    --------------------------------------------------------------------------

    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TOP")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1P")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2P")

    -- Register the stone gate set keep root wooden floor whith overhanging part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOTP_PART",
	Name = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOTP_NAME",
	Description = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOTP_DESC",
	Category = "CORE",
        BuildingZone = _OverHangingBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2P_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1P_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _stoneCost*_NoSandbox }
		}
	},
        IsVisibleWhenBuilt = false,  -- to mask the green diamond
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 8,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, _nameAsset .. "_ROOTP")
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_0_".. _nameFbx .. "_RootPart", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })
end

registerRootPart("baseOverHanging", "keep", "KeepH", "KEEPH", 2)
registerRootPart("baseOverHanging", "keep", "KeepI", "KEEPI", 2)
registerRootPart("baseOverHanging", "keep", "KeepJ", "KEEPJ", 2)
registerRootPart("baseOverHanging", "keep", "KeepK", "KEEPK", 2)
registerRootPart("baseOverHanging", "keep", "KeepL", "KEEPL", 2)
