local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/domeRoofs/domeCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})


-- Register Circular Tower materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_CIRCULAR_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/roofs/domeRoofs/domeCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_CIRCULAR_DOME_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_CIRCULAR_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartCircularTower.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_CIRCULAR_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_CIRCULAR_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_CIRCULAR_WHITE_STONE_FLOOR_GROUND_MATERIAL")

-- Register Circular Tower prefab nodes
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_3_Circular_TopPart", StoneGateSetPrefix .. "_CIRCULAR_TOP_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_2_Circular_Tiling2Part", StoneGateSetPrefix .. "_CIRCULAR_TILING2_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_RootBPart", StoneGateSetPrefix .. "_CIRCULAR_ROOTB_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_1_Circular_Tiling1BPart", StoneGateSetPrefix .. "_CIRCULAR_TILING1B_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_2_Circular_Tiling2BPart", StoneGateSetPrefix .. "_CIRCULAR_TILING2B_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_RootHALFWHITEPart", StoneGateSetPrefix .. "_CIRCULAR_ROOTHALFWHITE_PREFAB")
StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_1_Circular_Tiling1HALFWHITEPart", StoneGateSetPrefix .. "_CIRCULAR_TILING1HALFWHITE_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_CIRCULAR_TOP",
	StoneGateSetPrefix .. "_CIRCULAR_TILING2",
	StoneGateSetPrefix .. "_CIRCULAR_TILING2B"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerCircularTower(_option, _optionBase)
    -- Register Circular Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_Root" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_CIRCULAR_ROOT" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_1_Circular_Tiling1" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_CIRCULAR_TILING1" .. _optionBase .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_CIRCULAR_TILING1" .. _optionBase .. _option)

    -- Register the stone gate set circular tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CIRCULAR_ROOT" .. _optionBase .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_CIRCULAR_ROOT" .. _optionBase .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_CIRCULAR_ROOT" .. _optionBase .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _CircularBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_CIRCULAR_ROOT" .. _optionBase .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_CIRCULAR_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING1" .. _optionBase .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 6,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "CIRCULAR_ROOT" .. _optionBase .. _option)
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_Root" .. _optionBase .. _option .. "Part", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4,
        SoftRadius = 4,
        IsPermanent = true
    })
end

registerCircularTower("", "") -- with stone ground
registerCircularTower("B", "") -- with wooden ground
registerCircularTower("", "HALFWHITE") -- Half white with stone ground

-- roof ...
 
function registerRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _angle)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/circularTower/" .. _nameFbx .. _angle .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5*_NoSandbox }
		}
	}
    })
end

registerRoofPart( "roofs/simpleRoofs/roofCircularTower", "CircularRoof", "CIRCULAR_ROOF", "45" )
registerRoofPart( "roofs/simpleRoofs/roofCircularTower", "CircularRoof", "CIRCULAR_ROOF", "30" )
registerRoofPart( "roofs/simpleRoofs/roofCircularTower", "CircularRoof", "CIRCULAR_ROOF", "60" )
registerRoofPart( "roofs/domeRoofs/domeCircularTower", "CircularRoofDome", "CIRCULAR_ROOF_DOME", "" )

---- Ramparts...

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGate:registerAssetId("models/ramparts/rampartCircularTower.fbx/Prefab/circularTower/step_4_Circular_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	}
    })
end

registerRampartPart( "Rampart", "RAMPART" )
registerRampartPart( "RampartB", "RAMPART_B" )
