local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/bridge.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Squared Tower materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SST_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_SST_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_RAMPART_SST_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_SST_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/ramparts/bridge.fbx/Materials/murChateau", StoneGateSetPrefix .. "_BRIDGE_SST_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_SST_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_SST_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_RAMPART_SST_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/ramparts/bridge.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_BRIDGE_SST_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_SST_WHITE_STONE_FRAME_MATERIAL")

-- Register Small Squared Tower prefab nodes
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", StoneGateSetPrefix .. "_SST_TOP_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2Part", StoneGateSetPrefix .. "_SST_TILING2_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2PPart", StoneGateSetPrefix .. "_SST_TILING2P_PREFAB")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.05 }
	    }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_SST_TOP",
	StoneGateSetPrefix .. "_SST_TILING2",
	StoneGateSetPrefix .. "_SST_TILING2P"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerSmallSquaredTower(_option, _optionBase)
    -- Register Small Squared Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _SstBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	},
        VillagerCapacity = 2,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "SST_ROOT" .. _optionBase .. _option)
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _optionBase .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
    })
end

registerSmallSquaredTower("", "") -- with stone ground
registerSmallSquaredTower("P", "") -- with wooden ground
registerSmallSquaredTower("", "HALFWHITE") -- Half white with stone ground

function registerSmallSquaredGateTower(_option, _optionBase)
    -- Register Small Squared Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _SstGateBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1" .. _optionBase .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	}
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _optionBase .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
    })
end

registerSmallSquaredGateTower("", "GATE") -- Gate with stone ground
registerSmallSquaredGateTower("P", "GATE") -- Gate with wooden ground
registerSmallSquaredGateTower("", "GATE2") -- Gate2 with stone ground
registerSmallSquaredGateTower("P", "GATE2") -- Gate2 with wooden ground

function registerSmallSquaredStairTower(_option, _optionBase)
    -- Register Small Squared Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _optionBase .. "Part", StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. "_PREFAB")
    StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_Top" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_TOP" .. _optionBase .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TOP" .. _optionBase .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _SstStairBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT" .. _optionBase .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP" .. _optionBase .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0*_NoSandbox }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	}
    })
end

registerSmallSquaredStairTower("", "STAIR") -- Stair with stone ground
registerSmallSquaredStairTower("P", "STAIR") -- Stair with wooden ground

-- Roofs...
 
function registerRoofPart(_nameFbx, _nameAsset, _buildingPartType)
    StoneGate:registerAssetId("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 3*_NoSandbox }
		}
	}
    })

    StoneGate:registerPrefabComponent("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerRoofPart( "RoofA45", "ROOF_A45", "MAJOR" )
registerRoofPart( "RoofB45", "ROOF_B45", "MAJOR" )
registerRoofPart( "RoofD45", "ROOF_D45", "MAJOR" )
registerRoofPart( "RoofE45", "ROOF_E45", "MAJOR" )
registerRoofPart( "RoofF45", "ROOF_F45", "MAJOR" )
registerRoofPart( "RoofG45", "ROOF_G45", "MAJOR" )
registerRoofPart( "RoofH45", "ROOF_H45", "MINOR" )
registerRoofPart( "RoofI45", "ROOF_I45", "MINOR" )
registerRoofPart( "RoofJ45", "ROOF_J45", "MAJOR" )
registerRoofPart( "RoofK45", "ROOF_K45", "MAJOR" )
registerRoofPart( "RoofL45", "ROOF_L45", "MINOR" )
registerRoofPart( "RoofA30", "ROOF_A30", "MAJOR" )
registerRoofPart( "RoofB30", "ROOF_B30", "MAJOR" )
registerRoofPart( "RoofD30", "ROOF_D30", "MAJOR" )
registerRoofPart( "RoofE30", "ROOF_E30", "MAJOR" )
registerRoofPart( "RoofF30", "ROOF_F30", "MAJOR" )
registerRoofPart( "RoofG30", "ROOF_G30", "MAJOR" )
registerRoofPart( "RoofH30", "ROOF_H30", "MINOR" )
registerRoofPart( "RoofI30", "ROOF_I30", "MINOR" )
registerRoofPart( "RoofJ30", "ROOF_J30", "MAJOR" )
registerRoofPart( "RoofK30", "ROOF_K30", "MAJOR" )
registerRoofPart( "RoofL30", "ROOF_L30", "MINOR" )
registerRoofPart( "RoofA60", "ROOF_A60", "MAJOR" )
registerRoofPart( "RoofB60", "ROOF_B60", "MAJOR" )
registerRoofPart( "RoofD60", "ROOF_D60", "MAJOR" )
registerRoofPart( "RoofE60", "ROOF_E60", "MAJOR" )
registerRoofPart( "RoofF60", "ROOF_F60", "MAJOR" )
registerRoofPart( "RoofG60", "ROOF_G60", "MAJOR" )
registerRoofPart( "RoofH60", "ROOF_H60", "MINOR" )
registerRoofPart( "RoofI60", "ROOF_I60", "MINOR" )
registerRoofPart( "RoofJ60", "ROOF_J60", "MAJOR" )
registerRoofPart( "RoofK60", "ROOF_K60", "MAJOR" )
registerRoofPart( "RoofL60", "ROOF_L60", "MINOR" )

---- Ramparts...

function registerRampartPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType, _category)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	}
    })

    StoneGate:registerPrefabComponent("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartA", "RAMPART_A", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartB", "RAMPART_B", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartC", "RAMPART_C", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartD", "RAMPART_D", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartE", "RAMPART_E", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/bridge", "Bridge1", "BRIDGE1", "MINOR", "EXTENSION" )
registerRampartPart( "ramparts/bridge", "Bridge1P", "BRIDGE1P", "MINOR", "EXTENSION" )
registerRampartPart( "ramparts/bridge", "HalfArch1", "HALF_ARCH1", "MINOR", "EXTENSION" )
registerRampartPart( "ramparts/bridge", "HalfArch2", "HALF_ARCH2", "MINOR", "EXTENSION" )
registerRampartPart( "basic/basicSmallSquaredTower", "StoneFloor", "STONE_FLOOR", "MAJOR", "EXTENSION" )
registerRampartPart( "basic/basicSmallSquaredTower", "StoneFloorP", "STONE_FLOORP", "MAJOR", "EXTENSION" )

-- platforms
function configureStairPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType, _category)
    registerRampartPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType, _category)
    StoneGate:configurePrefabFlagList("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", { "PLATFORM" })
end
configureStairPart( "ramparts/rampartSmallSquaredTower", "StairA", "STAIR_A", "MAJOR", "EXTENSION" )
configureStairPart( "ramparts/rampartSmallSquaredTower", "StairB", "STAIR_B", "MAJOR", "EXTENSION" )
configureStairPart( "ramparts/rampartSmallSquaredTower", "Door_StairC", "STAIR_C", "DOOR", "DOOR" )
configureStairPart( "ramparts/rampartSmallSquaredTower", "Door_StairD", "STAIR_D", "DOOR", "DOOR" )
StoneGate:configurePrefabFlagList("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopSTAIRPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopSTAIRPPart", { "PLATFORM" })

function registerSmallSquaredStairEntranceTower(_option, _optionBase)
    -- Register Small Squared Tower prefab nodes
    StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_RootDoorStair" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT_STAIR" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Door_Stair" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_DOOR_STAIR" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_TilingStair" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING_STAIR" .. _optionBase .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopStair" .. _optionBase .. _option .. "Part", StoneGateSetPrefix .. "_SST_TOP_STAIR" .. _optionBase .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TOP_STAIR" .. _optionBase .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_DOOR_STAIR" .. _optionBase .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING_STAIR" .. _optionBase .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT_STAIR" .. _optionBase .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT_STAIR" .. _optionBase .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT_STAIR" .. _optionBase .. _option .. "_DESC",
	Category = "DOOR",
	BuildingZone = _SstStairBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT_STAIR" .. _optionBase .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP_STAIR" .. _optionBase .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING_STAIR" .. _optionBase .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_DOOR_STAIR" .. _optionBase .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0*_NoSandbox }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	}
    })

    StoneGate:registerPrefabComponent("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_RootDoorStair" .. _optionBase .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR"
    })
    StoneGate:configurePrefabFlagList("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopStair" .. _optionBase .. _option .. "Part", { "PLATFORM" })
end

registerSmallSquaredStairEntranceTower("", "E") -- Plain stairs with stone ground
registerSmallSquaredStairEntranceTower("P", "E") -- Plain stairs with wooden ground
