local drawBridge = ...

DrawBridgePrefix = StoneGateSetPrefix .. "_DRAW_BRIDGE"

drawBridge:dofile("scripts/compBridgeOpenClose.lua")
drawBridge:dofile("scripts/compPortcullisOpenClose.lua")

drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeA.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeB.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeC.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/portcullis.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Draw Bridges materials
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Materials/murChateau", StoneGateSetPrefix .. "_DRAWBRIDGE_A_STONE_MANOR_WALL_MATERIAL")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_DRAWBRIDGE_A_WHITE_STONE_FRAME_MATERIAL")
drawBridge:registerAssetId("models/drawBridges/drawBridgeB.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_DRAWBRIDGE_B_WHITE_STONE_FRAME_MATERIAL")
drawBridge:registerAssetId("models/drawBridges/drawBridgeC.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_DRAWBRIDGE_C_WHITE_STONE_FRAME_MATERIAL")

-- Register bridge tiling (the pillar part that extends to the ground)
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/TilingPart", DrawBridgePrefix .. "_TILING_PREFAB")
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_TILING_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_TILING_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
})

function registerExtremityPart(_nameFbx, _extremity, _upperAttachType, _lowerAttachType)
    -- Register bridge extremity part
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part", DrawBridgePrefix .. "_" .. _extremity .. "_PREFAB")
    drawBridge:register({
	    DataType = "BUILDING_PART",
	    Id = DrawBridgePrefix .. "_" .. _extremity .. "_PART",
	    ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = DrawBridgePrefix .. "_" .. _extremity .. "_PREFAB",
		SlopePartList = {
			DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PART"
		},
		CoreBasementPartList = {
			DrawBridgePrefix .. "_TILING_PART"
		},
		UpperAttachType = _upperAttachType,
		LowerAttachType = _lowerAttachType            },
	    Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	    }
    })

    -- Register bridge slope part
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "SlopePart", DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PREFAB",
		FillerList = {
			DrawBridgePrefix .. "_TILING_PART"
		}
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
        }
    })

    -- Configure BRIDGE colliders
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part", { "BRIDGE" })
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "SlopePart", { "BRIDGE" })

    -- Configure pillars zone
    drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part/pillar" .. _nameFbx .. "Zone", { DataType = "COMP_RIGID_BODY", Mass = 0 })
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part/pillar" .. _nameFbx .. "Zone", { "TERRAIN" })

end

registerExtremityPart("Start", "START", ATTACH_NODE_TYPE.RIGHT, ATTACH_NODE_TYPE.LEFT)
registerExtremityPart("End", "END", ATTACH_NODE_TYPE.LEFT, ATTACH_NODE_TYPE.RIGHT)

function registerCenterParts(_nameFbx, _option)
    -- Register bridge center top part
    drawBridge:registerAssetId("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", DrawBridgePrefix .. "_" .. _option .. "_CENTER_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CENTER_TOP_PART",
        BuildingZone = _DrawBridgeBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_" .. _option .. "_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "TOOL", Quantity = 2*_NoSandbox },
			{ Resource = "PLANK", Quantity = 10*_NoSandbox }
                }
        }
    })
    if _option == "C" then
        drawBridge:registerPrefabComponent("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", {
	    DataType = DrawBridgePrefix .. "_COMP",
            DrawBridge = _option,
	    MinDistanceVillager = 100,
	    MaxDistanceVillager = 150,
	    MinDistanceShip = 150,
	    MaxDistanceShip = 200,
            HasBoat = _flagHarbourV2,
	    Enabled = true
        })
        drawBridge:registerPrefabComponent("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart/herse1_sgv5db" .. _nameFbx, {
            DataType = "STONE_GATE_PORTCULLIS_COMP",
            DeltaElevation = -3,
            MinDistanceVillager = 50,
            SpeedMove = 2,
            Enabled = true
        })
    else
        drawBridge:registerPrefabComponent("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", {
	    DataType = DrawBridgePrefix .. "_COMP",
            DrawBridge = _option,
	    MinDistanceVillager = 50,
	    MaxDistanceVillager = 100,
	    MinDistanceShip = 100,
	    MaxDistanceShip = 150,
            HasBoat = _flagHarbourV2,
	    Enabled = true
        })
    end
    drawBridge:registerPrefabComponent("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart/herse_sgv5db" .. _nameFbx, {
        DataType = "STONE_GATE_PORTCULLIS_COMP",
        DeltaElevation = -3,
        MinDistanceVillager = 50,
        SpeedMove = 2,
        Enabled = true
    })

    -- Register the center root part (the scalable part that places multiple center top parts)
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/RootPart", DrawBridgePrefix .. "_CENTER_ROOT_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CENTER_CORE_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = DrawBridgePrefix .. "_CENTER_ROOT_PREFAB",
		EndPart = DrawBridgePrefix .. "_" .. _option .. "_CENTER_TOP_PART",
		FillerList = {
			DrawBridgePrefix .. "_TILING_PART"
		}
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "PLANK", Quantity = 1*_NoSandbox }
                }
        }
    })

    -- Register the core part (the part with the bridge mover)
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CORE_PART",
	Name = DrawBridgePrefix .. "_" .. _option .. "_CORE_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = DrawBridgePrefix .. "_START_PART",
		EndPart = DrawBridgePrefix .. "_END_PART",
		FillerList = {
			DrawBridgePrefix .. "_" .. _option .. "_CENTER_CORE_PART"
		},
		MinimumScale = 1,
                MinimumHeightAboveGround = 0,
                MiniatureConfig = {
                    ClipUnderGround = true,
                    GroupHeight = 0.0,
                    CameraPosition = { -2, 0, -20 },
                    OrientationOffset = 0,
                    CameraPitchOffset = 20.
                }
	},
	AssetBuildingFunction = DrawBridgePrefix .. "_FUNCTION"
    })

    -- Configure BRIDGE collider
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", { "BRIDGE" })
end

registerCenterParts("a", "A") -- simple draw bridge
registerCenterParts("b", "B") -- draw bridge with arrows
registerCenterParts("c", "C") -- double draw bridge

drawBridge:register({
		DataType = "BUILDING_FUNCTION_BRIDGE",
                Id = DrawBridgePrefix .. "_FUNCTION"
})

-- Register the additional portcullis part
drawBridge:registerAssetId("models/drawBridges/portcullis.fbx/Prefab/drawBridge/PortcullisPart", DrawBridgePrefix .. "_PORTCULLIS_PREFAB")
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_PORTCULLIS_PART",
	Name = DrawBridgePrefix .. "_PORTCULLIS_NAME",
	Description = DrawBridgePrefix .. "_PORTCULLIS_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_PORTCULLIS_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "TOOL", Quantity = 1*_NoSandbox }
		}
	}
})
drawBridge:registerPrefabComponent("models/drawBridges/portcullis.fbx/Prefab/drawBridge/PortcullisPart", {
    DataType = "STONE_GATE_PORTCULLIS_COMP",
    DeltaElevation = -3,
    MinDistanceVillager = 50,
    SpeedMove = 2,
    Enabled = true
})

