local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/core.fbx", {
    DataType = "BUILDING_ASSET_PROCESSOR"
})

StoneGate:registerAssetId("models/core.fbx/Prefab/CoreBuildingPart", StoneGateSetPrefix .. "_CORE_BUILDING_PREFAB")

-- Register the Core part
StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CORE_BUILDING_PART",
	Name = StoneGateSetPrefix .. "_CORE_BUILDING_NAME",
	Description = StoneGateSetPrefix .. "_CORE_BUILDING_DESC",
	Category = "CORE",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_CORE_BUILDING_PREFAB"
	},
        IsVisibleWhenBuilt = false
})


    _MainBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {8, 12} ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _KeepBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {8, 8} ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _SstBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {4, 4} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( {4, 2} ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( {2, 4} ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _SstStairBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {4, 4} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( {2, 4} ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _SstGateBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 4, 4}, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( { 4, 1 }, { 0, 1.5 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 4, 1 }, { 0, -1.5 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 4 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    local OtowerVarTemp = 2/math.tan(math.pi/8)
    _OTowerBuildingZone = { ZoneEntryList = {
            { Polygon = {
                  {-2, OtowerVarTemp},
                  {2, OtowerVarTemp},
                  {OtowerVarTemp, 2},
                  {OtowerVarTemp, -2},
                  {2, -OtowerVarTemp},
                  {-2, -OtowerVarTemp},
                  {-OtowerVarTemp, -2},
                  {-OtowerVarTemp, 2} },
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _LGate2BuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 8, 2 }, { 0, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 8, 2 }, { 0, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 8, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _LGate2_2BuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 8, 2 }, { 0, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 8 }, { -3, 0 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { 3, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 8, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _LGate3BuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 8, 2 }, { 0, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { -3, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { 3, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 8, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _LGate4BuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 2, 2 }, { -3, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { 3, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { -3, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 2 }, { 3, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 8, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _SmallGateBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 4, 8}, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( { 4, 2 }, { 0, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 4, 2 }, { 0, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _SmallGateExtensionBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 4, 8 }, { 0, 0 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( { 2, 8 }, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _SmallGateHalfRoofBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 4, 8}, { 0, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( { 4, 2 }, { 0, 3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 4, 2 }, { 0, -3 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false } },
            { Polygon = polygon.createRectangle( { 2, 8 }, { -1, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _SmallGateExtensionHalfRoofBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 4, 8 }, { 0, 0 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( { 2, 8 }, { -1, 0 } ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false } }
    }}
    _DrawBridgeBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( { 2, 2 }, { 0, 0 } ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
    }}
    local Angle45VarTemp = 2*math.tan(math.pi/8)
    _Angle45BuildingZone = { ZoneEntryList = {
            { Polygon = {
                  {Angle45VarTemp+1, 2},
                  {Angle45VarTemp+1, -2},
                  {-Angle45VarTemp+1, -2},
                  {-4*math.cos(math.pi/4)+Angle45VarTemp+1, -4*math.sin(math.pi/4)+2} },    
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _CircularBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createCircle( 3.3, { 0, 0 }, 16 ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _SmallCircularBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createCircle( 1.5, { 0, 0 }, 24 ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _OverHangingBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {8, 8} ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
    }}
    _buildingZonePillings = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {1.75, 1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {-1.75, 1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {-1.75, -1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {1.75, -1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {4, 2} ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false }},
            { Polygon = polygon.createRectangle( {2, 4}),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false }} -- TBC: move to the wodenFloor Decoration D (in case of wooden bridge only)
        }
    }
    _buildingZoneWoodenStair = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {1.75, 1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {-1.75, 1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {-1.75, -1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {0.5, 0.5}, {1.75, -1.75} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true }},
            { Polygon = polygon.createRectangle( {2, 4} ),
              Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = false }}
        }
    }

-- Function list
    function overrideWorkplaceName(_name)
        StoneGate:override({
                Id = "BUILDING_FUNCTION_" .. _name,
                Name = StoneGateSetPrefix .. "_BUILDING_FUNCTION_" .. _name .. "_NAME"
        })
    end
-- Waiting for dummy exposed   overrideWorkplaceName("WOODEN_KEEP")
    overrideWorkplaceName("BAKERY")
    overrideWorkplaceName("BLACKSMITH")
    overrideWorkplaceName("BUILDER_WORKSHOP")
    overrideWorkplaceName("CHEESE_MAKER")
    overrideWorkplaceName("COOPERAGE")
    overrideWorkplaceName("JEWELER_WORKSHOP")
    overrideWorkplaceName("SAWMILL")
    overrideWorkplaceName("STONEMASON_HUT")
    overrideWorkplaceName("TAILOR_WORKSHOP")
    overrideWorkplaceName("WEAPONSMITH")
    overrideWorkplaceName("WEAVER_HUT")
    overrideWorkplaceName("BREWERY")
    overrideWorkplaceName("IRON_SMELTER")
    overrideWorkplaceName("GOLD_SMELTER")
    overrideWorkplaceName("GLASS_SMELTER")
    overrideWorkplaceName("HOP_FARM")
--    overrideWorkplaceName("VINEYARD")
    overrideWorkplaceName("WHEAT_FARM")
    overrideWorkplaceName("WINDMILL")

function registerAssignableFunction(_assetPrefix, _assetId)
    StoneGate:register({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _assetPrefix .. "_" .. _assetId .. "_FUNCTION_LIST",
                AssetFunctionList = {
                        "BUILDING_FUNCTION_GREAT_HALL",
                        "BUILDING_FUNCTION_BAILIFF_OFFICE",
                        "BUILDING_FUNCTION_TREASURY",
                        "BUILDING_FUNCTION_TAX_OFFICE",
--                        "BUILDING_FUNCTION_WOODEN_KEEP",
                        "BUILDING_FUNCTION_BAKERY",
                        "BUILDING_FUNCTION_BLACKSMITH",
                        "BUILDING_FUNCTION_BUILDER_WORKSHOP",
                        "BUILDING_FUNCTION_CHEESE_MAKER",
                        "BUILDING_FUNCTION_COOPERAGE",
                        "BUILDING_FUNCTION_JEWELER_WORKSHOP",
                        "BUILDING_FUNCTION_SAWMILL",
                        "BUILDING_FUNCTION_STONEMASON_HUT",
                        "BUILDING_FUNCTION_TAILOR_WORKSHOP",
                        "BUILDING_FUNCTION_WEAPONSMITH",
                        "BUILDING_FUNCTION_WEAVER_HUT",
                        "BUILDING_FUNCTION_FOOD_MARKET",
                        "BUILDING_FUNCTION_GOODS_MARKET",
                        "BUILDING_FUNCTION_LUXURY_MARKET",
                        "BUILDING_FUNCTION_SERVICE_COUNTER",
                        "BUILDING_FUNCTION_BREWERY",
                        "BUILDING_FUNCTION_IRON_SMELTER",
                        "BUILDING_FUNCTION_GOLD_SMELTER",
                        "BUILDING_FUNCTION_GLASS_SMELTER",
                        registerFunctionHouse(_assetPrefix, _assetId, "1"),
                        registerFunctionHouse(_assetPrefix, _assetId, "2"),
                        "BUILDING_FUNCTION_HOP_FARM",
--                        "BUILDING_FUNCTION_VINEYARD",
                        "BUILDING_FUNCTION_WHEAT_FARM",
                        "BUILDING_FUNCTION_WINDMILL",
                }
    })
    StoneGate:register({
		DataType = "BUILDING_FUNCTION_ASSIGNABLE",
                Id = _assetPrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION",
                AuthorizedFunctionList = _assetPrefix .. "_" .. _assetId .. "_FUNCTION_LIST"
    })
    return _assetPrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION"
end
 
-- Register all stone gate set building parts and their properties
StoneGate:dofile("scripts/basicMain.lua")
StoneGate:dofile("scripts/basicKeep.lua")
StoneGate:dofile("scripts/basicLargeGate.lua")
StoneGate:dofile("scripts/basicSmallSquaredTower.lua")
StoneGate:dofile("scripts/basicOctogonalTower.lua")
StoneGate:dofile("scripts/basicSmallGate.lua")
StoneGate:dofile("scripts/basicAngle45.lua")
StoneGate:dofile("scripts/hoarding.lua")
StoneGate:dofile("scripts/machicolation.lua")
StoneGate:dofile("scripts/decoration.lua")
StoneGate:dofile("scripts/drawBridge.lua")
StoneGate:dofile("scripts/basicCircularTower.lua")
StoneGate:dofile("scripts/basicSmallCircularTower.lua")
StoneGate:dofile("scripts/pointedRoof.lua")
StoneGate:dofile("scripts/woodenFloor.lua")
StoneGate:dofile("scripts/woodenFloorWhite.lua", StoneGateSetPrefixWhiteFloor)
StoneGate:dofile("scripts/baseOverHanging.lua")
StoneGate:dofile("scripts/pillings.lua")
StoneGate:dofile("scripts/assemblage.lua", AssemblagePrefix)

StoneGate:register({
    Id = StoneGateSetPrefix .. "_MONUMENT",
    DataType = "BUILDING",
    AssetCoreBuildingPart = StoneGateSetPrefix .. "_CORE_BUILDING_PART",
    Name = StoneGateSetPrefix .. "_MONUMENT_NAME",
    Description = StoneGateSetPrefix .. "_MONUMENT_DESC",
    BuildingType = "MODS",
    BuildingPartSetList = configureBuildingPartSetList(),
    MaterialSetList = configureMaterialSetList(configureMaterialSet),
    IsEditable = true
})

--[[ currently inhibited
for i,buildingPartSet in ipairs(configureBuildingPartSetList()) do
    StoneGate:override({
        Id = "MONUMENT_WOODEN_KEEP",
        BuildingPartSetList = {
                Action = "APPEND",
                buildingPartSet
        }
    })
end
    StoneGate:override({
        Id = "MONUMENT_WOODEN_KEEP",
        MaterialSetList = configureMaterialSetList(configureMaterialSet)
    })
--]]
