local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/woodenFloors/pillings.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix, _isVisibleWhenBuilt )
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
        	Name = _nodePrefix .. "_NAME",
	        Description = _nodePrefix .. "_DESC",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 0*_NoSandbox }
		    }
	        },
                IsVisibleWhenBuilt = _isVisibleWhenBuilt  -- false to mask the green diamond and to add a comment in preview
	})
end

function registerPrefabAndParts(_nameFbxFile, _nameModel, _nameFbx, _nameAsset)
    -- Register common prefab nodes
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_0_".. _nameFbx .. "_RootPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOT_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_1_".. _nameFbx .. "_Tiling1Part", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_2_".. _nameFbx .. "_Tiling2Part", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _nameFbx .. "_TopPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TOP_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _nameFbx .. "_StairPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_STAIR_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_0_".. _nameFbx .. "_RootFPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_ROOTF_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_1_".. _nameFbx .. "_Tiling1FPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1F_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_2_".. _nameFbx .. "_Tiling2FPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2F_PREFAB")
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _nameFbx .. "_StairFPart", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_STAIRF_PREFAB")
    -- Register common building parts
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TOP", false) -- mask the green diamond
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_STAIR", false) -- mask the green diamond
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1", true)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2", true)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_STAIRF", false) -- mask the green diamond
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING1F", true)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset .. "_TILING2F", true)
end
registerPrefabAndParts("pillings", "woodenFloor", "WFDecorationA", "PILLINGS")

function registerRootPart(_nameAsset, _plankCost, _option, _top, _tiling2, _tiling1, _root, _buildingZone)
    -- Register the root of a scaleable wooden part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" ..  _nameAsset .. _option .. "_" .. _root .. "_PART",
	Name = StoneGateSetPrefix .. "_" ..  _nameAsset .. _option .. "_" .. _root .. "_NAME",
	Description = StoneGateSetPrefix .. "_" ..  _nameAsset .. _option .. "_" .. _root .. "_DESC",
	Category = "CORE",
        BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_" .. _root .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_" ..  _nameAsset .. "_" .. _top .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_" ..  _nameAsset .. "_" .. _tiling2 .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_" ..  _nameAsset .. "_" .. _tiling1 .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _plankCost*_NoSandbox }
		}
	},
        IsVisibleWhenBuilt = false  -- to mask the green diamond
    })
end

registerRootPart("PILLINGS", 1, "A", "TOP", "TILING2", "TILING1", "ROOT", _buildingZonePillings)
registerRootPart("PILLINGS", 2, "B", "STAIR", "TILING2", "TILING1", "ROOT", _buildingZoneWoodenStair)
registerRootPart("PILLINGS", 2, "F", "STAIRF", "TILING2F", "TILING1F", "ROOTF", _buildingZoneWoodenStair)
function configurePlatformPart(_nameFbxFile, _nameModel, _nameFbx)
    StoneGate:configurePrefabFlagList("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _nameFbx .. "Part", { "PLATFORM" })
end
configurePlatformPart("pillings", "woodenFloor", "WFDecorationA_Stair")
configurePlatformPart("pillings", "woodenFloor", "WFDecorationA_StairF")
StoneGate:registerPrefabComponent("models/woodenFloors/pillings.fbx/Prefab/woodenFloor/step_0_WFDecorationA_RootFPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR"
})

function registerDefaultDecoration(_nameFbxFile, _nameModel, _nameFbx, _nameAsset)
    StoneGate:registerAssetId("models/woodenFloors/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/".. _nameFbx .. "Part", StoneGateSetPrefix .. "_" ..  _nameAsset .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_" ..  _nameAsset, true) -- no special comment in the preview
end
registerDefaultDecoration("pillings", "woodenFloor", "WFDecorationH", "PILLINGSB")